/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
          y
          |
          |
          | v3              v2=(1,1,0)
        1 +-----------------+
          |               / |
          |             /   |
          |           /     |
          |         /       |
          |       /         |
          |     /           |
          |   /             |
          | /               | v1
       v0 +-----------------+-----> x
          0                 1

*/
public class InterpolateColor
{
   public static void main(String[] args)
   {
      final int SIZE = 256;

      // Create a Model object to hold the geometry.
      Model model = new Model();
      model.doBackFaceCulling = false;

      // Add the geometry to the Model.
      Vertex v0 = new Vertex(0.0, -0.5, 0.0);
      Vertex v1 = new Vertex(0.0,  0.5, 0.0);
      Vertex v2 = new Vertex(1.0,  0.5, 0.0);
      Vertex v3 = new Vertex(1.0, -0.5, 0.0);

      v0.setColor(Color.green);
      v1.setColor(Color.blue);
      v2.setColor(Color.blue);
      v3.setColor(Color.green);

      model.addTriangle(v0, v1, v2);
      model.addTriangle(v0, v2, v3);

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Set up the camera's location.
      scene.camera.viewTranslate(0, 0, 1.5);

      // Set up the camera's view volume.
      scene.camera.projPerspective(-0.5, 0.5, -0.5, 0.5, 0.5);
      //scene.camera.projOrtho(-1, 1, -0.5, 0.5, 0.5);

      // Add the Model to the Scene.
      scene.positionList.add(new Position(model));

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      //int vp_width  = 1024; // for the orthographic projection
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a background color.
      fb.clearFB(Color.darkGray);

//      Pipeline.debug = true;
//      Rasterize.debug = true;

      // Render our scene into the frame buffer.
      Pipeline.render(scene, fb);
      fb.dumpFB2File( String.format("PPM_ColorSquare_Frame%02d.ppm", 0) );

      // Spin the model 360 degrees.
      for (int i = 1; i < 360; i++)
      {
         // Rotate the model by 1 degree.
         scene.positionList.get(0).rotateY(1);

         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_ColorSquare_Frame%03d.ppm", i));
      }
   }
}