/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;
import java.io.File;

/**
   This program shows that our renderer does
   not keep track of any depth information.

   This program draws two models, a sphere and
   a cow. The cow is inside the sphere. But the
   final image drawn by the renderer depends on
   what order the models are placed in the Scene's
   model list. Whatever model is second in the list
   is drawn over the model that is first in the list.
   So if the cow is second in the list, it gets drawn
   over the sphere and does not appear to be inside
   the sphere.

   The final image drawn by the renderer should not
   depend on the order in which models are placed in
   the scene's model list. The final image should
   instead depend on the order of the models in space.
*/
public class SomethingWrong3
{
   public static void main(String[] args)
   {
      Position p1 = new Position( new Sphere(1.0, 30, 30) );
      p1.model.setColor(Color.red);
      p1.translate(0, 0, -2);

      Position p2 = new Position( new ObjSimpleModel(new File("../../assets/cow.obj")) );
      p2.model.setColor(Color.blue);
      // Place the cow in the middle of the sphere.
      p2.translate(0, 0, -2);


      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Add the Models to the Scene.
      scene.addPosition(p1, p2);


      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Give the framebuffer a background color.
      fb.clearFB(java.awt.Color.black);
      // Render our scene into the frame buffer.
      Pipeline.render(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( "SomethingWrong3a.ppm" );

      // Reverse the order of the two models in the scene's position list.
      Position temp = scene.positionList.remove(0);
      scene.addPosition(temp);

      // Give the framebuffer a background color.
      fb.clearFB(java.awt.Color.black);
      // Render our scene into the frame buffer.
      Pipeline.render(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( "SomethingWrong3b.ppm" );
   }
}
