/*

*/

import renderer.scene.*;
import renderer.models_F.Sphere;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.awt.Color;
import java.awt.event.*;

/**

*/
@SuppressWarnings("serial")
public class SphereDemo extends AnimationFrame
{
   private Scene scene;
   private double rotation = 0.0;
   private double delta = 1.0;
   private boolean frontFacingIsCCW = true;

   private double xRotation = 0.0;
   private double yRotation = 0.0;
   private double zRotation = 0.0;

   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
   */
   public SphereDemo(String title, int fbWidth, int fbHeight, int fps)
   {
      super(title, fbWidth, fbHeight, fps);

      int k = 15;
      int n = 15;

      // Create a Model object that holds the geometry.
      Model model = new Sphere(1.0, k, n);

      // Give the Model a color.
      model.setColor(new Color(255, 0, 255));

      // Create the Scene object that we shall render
      scene = new Scene();

      // Add the Model to the Scene.
      scene.addPosition(new Position(model));

      // Set up the Camera's view frustum.
      double fov    = 25.0;
      double aspect = 1.0;
      double near   = 1.0;
      scene.camera.projPerspective(fov, aspect, near);

      // Set up the Camera's location and orientation.
      scene.camera.viewTranslate(0, 0, 5);
   }


   // Implement the ActionListener interface.
   @Override public void actionPerformed(ActionEvent e)
   {
      //System.out.println( e );

      rotation += delta;

      // Set up the Camera's location and orientation.
      scene.camera.view2Identity();
      scene.camera.viewRotateY(rotation);
      scene.camera.viewTranslate(0, 0, 5);

      Model model = scene.positionList.get(0).model;

      FrameBuffer fb = this.fbp.getFrameBuffer();
      fb.clearFB(Color.black);
      // Set a viewport within the framebuffer.
      fb.setViewport(0, 0, 400, 400); //upper-left-hand-corner, width, height
      // Give the viewport a background color.
      fb.clearVP(java.awt.Color.black);
      model.doBackFaceCulling = false;
    //model.facesHaveTwoSides = false;
      Pipeline.render(scene, fb);

      // Set a viewport within the framebuffer.
      fb.setViewport(400, 0, 400, 400); //upper-left-hand-corner, width, height
      // Give the viewport a background color.
      fb.clearVP(java.awt.Color.black);
      model.doBackFaceCulling = true;
    //model.facesHaveTwoSides = false;
      Pipeline.render(scene, fb);
/*
      // Set a viewport within the framebuffer.
      fb.setViewport(800, 0, 400, 400); //upper-left-hand-corner, width, height
      // Give the viewport a background color.
      fb.clearVP(java.awt.Color.black);
      model.doBackFaceCulling = false;
      model.facesHaveTwoSides = true;
      Pipeline.render(scene, fb);
*/
      fbp.update();
      repaint();
   }


   // Implement part of the KeyListener interface.
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      char c = e.getKeyChar();
      if ('h' == c)
      {
         print_help_message();
         return;
      }
      else if ('r' == c)
      {
         delta += 0.1;
         System.out.println("delta = " + delta);
      }
      else if ('R' == c)
      {
         delta -= 0.1;
         System.out.println("delta = " + delta);
      }
      else if ('f' == c)
      {
         int fps = getFPS();
         fps -= 1;
         if (0 == fps) fps = 1;
         setFPS(fps);
         System.out.println("fps = " + fps);
      }
      else if ('F' == c)
      {
         int fps = getFPS();
         fps += 1;
         setFPS(fps);
         System.out.println("fps = " + fps);
      }
      else if ('g' == c)
      {
         frontFacingIsCCW = ! frontFacingIsCCW;
         for (Position p : scene.positionList)
            p.model.frontFacingIsCCW = frontFacingIsCCW;
         System.out.print("Front face is ");
         System.out.println(frontFacingIsCCW ? "CCW" : "CW");
      }
      else if ('c' == c)
      {
         // Change the solid random color of the sphere model.
         scene.positionList.get(0).model.setColorRandom();
      }
      else if ('e' == c)
      {
         // Change the random color at each edge.
         Model m = scene.positionList.get(0).model;
         for (Primitive p : m.primitiveList)
            p.setColorRandom();
      }
      else if ('p' == c)
      {
         scene.camera.perspective = ! scene.camera.perspective;
      }
      else if ('x' == c)
      {
         xRotation -= 2.0;
      }
      else if ('X' == c)
      {
         xRotation += 2.0;
      }
      else if ('y' == c)
      {
         yRotation -= 2.0;
      }
      else if ('Y' == c)
      {
         yRotation += 2.0;
      }
      else if ('z' == c)
      {
         zRotation -= 2.0;
      }
      else if ('Z' == c)
      {
         zRotation += 2.0;
      }

      // Set the model-to-view transformation matrix.
      // The order of the transformations is very important!
      Position position = scene.positionList.get(0);
      position.matrix2Identity();
      // Move the model relative to its new position.
      position.rotateX(xRotation);
      position.rotateY(yRotation);
      position.rotateZ(zRotation);
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // Define initial dimensions for a FrameBuffer.
    //int width  = 1200;
      int width  = 800;
      int height = 400;
      // Create an InteractiveFrame containing a FrameBuffer
      // with the given dimensions. NOTE: We need to call the
      // SphereDemo constructor in the Java GUI Event
      // Dispatch Thread, otherwise we get a race condition
      // between the constructor (running in the main() thread)
      // and the very first ComponentEvent (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         new Runnable() // an anonymous inner class constructor
         {
            public void run() // implement the Runnable interface
            {
               // call the constructor that builds the gui
               (
                  new SphereDemo("Renderer 11", width, height, 30)
               ).setResizable(false);
            }
         }
      );
   }//main()


   private static void print_help_message()
   {
      System.out.println("Use the r/R keys to change the speed of rotation.");
      System.out.println("Use the f/F keys to slow down or speed up the frame rate.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, z/Z, keys to rotate the sphere around the x, y, z axes.");
      System.out.println("Use the 'c' key to change the solid model color.");
      System.out.println("Use the 'e' key to change the random edge colors.");
      System.out.println("Use the 'g' key to toggle CW vs CCW.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
