/*

*/

import renderer.scene.*;
import renderer.models_T.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.io.File;
import java.awt.Color;
import java.awt.event.*;
import java.util.ArrayList;

/**

*/
@SuppressWarnings("serial")
public class TestCulling extends InteractiveFrame
{
   private static boolean doBackFaceCulling = true;
   private static boolean frontFacingIsCCW = true;
//   private static boolean render_z_buffer = false;

   private static double eyeX = 0.0;
   private static double eyeY = 0.0;
   private static double eyeZ = 2.0;
   private static double eyeRotX = 0.0;
   private static double eyeRotY = 0.0;
   private static double eyeRotZ = 0.0;

   private boolean letterbox = false;
   private double fovy = 90;
   private double aspectRatio = 1.0;
   private double near   =  1.0;
   private double left   = -1.0;
   private double right  =  1.0;
   private double bottom = -1.0;
   private double top    =  1.0;

   private boolean showMatrix = false;
   private double xTranslation = 0.0;
   private double yTranslation = 0.0;
   private double zTranslation = 0.0;
   private double xRotation = 0.0;
   private double yRotation = 0.0;
   private double zRotation = 0.0;
   private double scale = 1.0;

   private Scene scene;

   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
   */
   public TestCulling(String title, int fbWidth, int fbHeight)
   {
      super(title, fbWidth, fbHeight);

      int count = 3;  // number of triangles along each edge

      Model m = new Model();

      double step = 2.0 / count;

      Vertex center  = new Vertex(0,  0,  0);

      double y = -1.0;
      double z = -1.0;
      for (int i = 0; i < count; i++)
      {
         // front and back edges
         m.addTriangle(new Vertex(0, y,  1), center, new Vertex(0, y+step,  1));
         m.addTriangle(new Vertex(0, y, -1), new Vertex(0, y+step, -1), center);
         // top and bottom edges
         m.addTriangle(new Vertex(0,  1, z), new Vertex(0,  1, z+step), center);
         m.addTriangle(new Vertex(0, -1, z), center, new Vertex(0, -1, z+step));
         y += step;
         z += step;
      }

      // Give the models random colors.
      for (Primitive p : m.primitiveList)
      {
         p.setColorRandom();
      }

      // Create the Scene object that we shall render
      scene = new Scene();

      // Set up the camera's location and orientation.
      scene.camera.viewTranslate(eyeX, eyeY, eyeZ);

      // Add a model to the Scene.
      scene.addPosition(new Position(m));
   }


   // Implement part of the KeyListener interface.
   @Override public void keyPressed(KeyEvent e)
   {
      //System.out.println( e );

      int keyCode = e.getKeyCode();
      // Only handle the four arrow keys.
      if (KeyEvent.VK_UP == keyCode   || KeyEvent.VK_DOWN == keyCode
       || KeyEvent.VK_LEFT == keyCode || KeyEvent.VK_RIGHT == keyCode)
      {
         if (KeyEvent.VK_UP == keyCode && e.getModifiers() != 0 )
         {
            eyeRotX += 1;
         }
         else if (KeyEvent.VK_DOWN == keyCode && e.getModifiers() != 0 )
         {
            eyeRotX -= 1;
         }
         else if (KeyEvent.VK_LEFT == keyCode && e.getModifiers() != 0 )
         {
            eyeRotY += 1;
         }
         else if (KeyEvent.VK_RIGHT == keyCode && e.getModifiers() != 0 )
         {
            eyeRotY -= 1;
         }
         else if (KeyEvent.VK_UP == keyCode)
         {
            eyeY += 0.1;
         }
         else if (KeyEvent.VK_DOWN == keyCode)
         {
            eyeY -= 0.1;
         }
         else if (KeyEvent.VK_LEFT == keyCode)
         {
            eyeX -= 0.1;
         }
         else if (KeyEvent.VK_RIGHT == keyCode)
         {
            eyeX += 0.1;
         }

         // Set up the camera's location and orientation.
         scene.camera.view2Identity();
         scene.camera.viewTranslate(eyeX, eyeY, eyeZ);
         scene.camera.viewRotateX(eyeRotX);
         scene.camera.viewRotateY(eyeRotY);

         if (showMatrix)
         {
            System.out.println("eyeRotX = " + eyeRotX
                           + ", eyeRotY = " + eyeRotY);
            System.out.print( scene.camera.viewMatrix );
         }

        // Render again.
         setupViewport();
      }
   }


   // Implement part of the KeyListener interface.
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      char c = e.getKeyChar();
      if ('h' == c)
      {
         print_help_message();
         return;
      }
      else if ('d' == c)
      {
         Pipeline.debug = ! Pipeline.debug;
       //Pipeline.debug_visible = ! Pipeline.debug_visible;
       //Rasterize.debug = ! Rasterize.debug;
      }
      else if ('g' == c)
      {
         doBackFaceCulling = ! doBackFaceCulling;
         for (Position p : scene.positionList)
            p.model.doBackFaceCulling = doBackFaceCulling;
         System.out.print("Back face culling is ");
         System.out.println(doBackFaceCulling ? "on" : "off");
      }
      else if ('G' == c)
      {
         frontFacingIsCCW = ! frontFacingIsCCW;
         for (Position p : scene.positionList)
            p.model.frontFacingIsCCW = frontFacingIsCCW;
         System.out.print("Front face is ");
         System.out.println(frontFacingIsCCW ? "CCW" : "CW");
      }
      else if (' ' == c)
      {
//         render_z_buffer = ! render_z_buffer;
      }
      else if ('l' == c)
      {
         letterbox = ! letterbox;
      }
      else if ('n' == c || 'N' == c)
      {
         // Move the near plane.
         if ('n' == c)
            near -= 0.01;
         else
            near += 0.01;
      }
      else if ('r' == c || 'R' == c)
      {
         // Change the aspect ratio.
         if ('r' == c)
            aspectRatio -= 0.1;
         else
            aspectRatio += 0.1;

         // Adjust right and left.
         // (Keep the vertical field-of-view fixed.)
         right =  top * aspectRatio;
         left  = -right;
      }
      else if ('o' == c || 'O' == c)
      {
         // Change left, right, bottom, and top.
         // (Keep the aspect ratio fixed.)
         if ('o' == c)
         {
            left   += 0.1 * aspectRatio;
            right  -= 0.1 * aspectRatio;
            bottom += 0.1;
            top    -= 0.1;
         }
         else
         {
            left   -= 0.1 * aspectRatio;
            right  += 0.1 * aspectRatio;
            bottom -= 0.1;
            top    += 0.1;
         }
      }
      else if ('e' == c)
      {
         // Change the random color at each triangle.
         Model m = scene.positionList.get(0).model;
         for (Primitive p : m.primitiveList)
            p.setColorRandom();
      }
      else if ('p' == c)
      {
         scene.camera.perspective = ! scene.camera.perspective;
      }
      else if ('m' == c)
      {
         showMatrix = ! showMatrix;
      }
      else if ('s' == c) // Scale the model 10% smaller.
      {
         scale /= 1.1;
      }
      else if ('S' == c) // Scale the model 10% larger.
      {
         scale *= 1.1;
      }
      else if ('x' == c)
      {
         xTranslation -= 0.1;
      }
      else if ('X' == c)
      {
         xTranslation += 0.1;
      }
      else if ('y' == c)
      {
         yTranslation -= 0.1;
      }
      else if ('Y' == c)
      {
         yTranslation += 0.1;
      }
      else if ('z' == c)
      {
         zTranslation -= 0.1;
      }
      else if ('Z' == c)
      {
         zTranslation += 0.1;
      }
      else if ('u' == c)
      {
         xRotation -= 2.0;
      }
      else if ('U' == c)
      {
         xRotation += 2.0;
      }
      else if ('v' == c)
      {
         yRotation -= 2.0;
      }
      else if ('V' == c)
      {
         yRotation += 2.0;
      }
      else if ('w' == c)
      {
         zRotation -= 2.0;
      }
      else if ('W' == c)
      {
         zRotation += 2.0;
      }

      // Set the model-to-view transformation matrix.
      // The order of the transformations is very important!
      Position position = scene.positionList.get(0);
      position.matrix2Identity();
      // Move the model relative to its new position.
      position.translate(xTranslation, yTranslation, zTranslation);
      position.rotateX(xRotation);
      position.rotateY(yRotation);
      position.rotateZ(zRotation);
      position.scale(scale);

      // Set up the camera's view volume.
      if (scene.camera.perspective)
      {
         scene.camera.projPerspective(left, right, bottom, top, near);
      }
      else
      {
         scene.camera.projOrtho(left, right, bottom, top);
      }
      if ('n'==c||'N'==c||'o'==c||'O'==c||'r'==c||'R'==c||'p'==c)
      {
         System.out.print( scene.camera );
      }

      if (showMatrix && ('m'==c
           ||'s'==c||'x'==c||'y'==c||'z'==c||'u'==c||'v'==c||'w'==c
           ||'S'==c||'X'==c||'Y'==c||'Z'==c||'U'==c||'V'==c||'W'==c))
      {
         System.out.println("xRot = " + xRotation
                        + ", yRot = " + yRotation
                        + ", zRot = " + zRotation);
         System.out.print( position.modelMatrix );
      }

      // Render again.
      setupViewport();
   }


   // Implement part of the ComponentListener interface.
   @Override public void componentResized(ComponentEvent e)
   {
      //System.out.println( e );

      // Get the new size of the FrameBufferPanel.
      int w = this.fbp.getWidth();
      int h = this.fbp.getHeight();

      // Create a new FrameBuffer that fits the new window size.
      FrameBuffer fb = new FrameBuffer(w, h);
      this.fbp.setFrameBuffer(fb);

      // Render again.
      setupViewport();
   }


   // Get in one place the code to set up the viewport.
   private void setupViewport()
   {
      // Render again.
      // Get the size of the FrameBuffer.
      FrameBuffer fb = this.fbp.getFrameBuffer();
      int w = fb.getWidthFB();
      int h = fb.getHeightFB();
      // Create a viewport with the correct aspect ratio.
      if ( letterbox )
      {
         if ( aspectRatio <= w/(double)h )
         {
            int width = (int)(h*aspectRatio);
            int xOffset = (w - width)/2;
            fb.setViewport(xOffset, 0, width, h);
         }
         else
         {
            int height = (int)(w/aspectRatio);
            int yOffset = (h - height)/2;
            fb.setViewport(0, yOffset, w, height);
         }
         fb.clearFB(Color.darkGray);
         fb.clearVP(Color.black);
      }
      else // the viewport is the whole framebuffer
      {
         fb.clearFB(Color.black);
         fb.setViewport();
      }
      Pipeline.render(scene, fb);
/*
      if ( render_z_buffer )
      {
         FrameBuffer zBuf = this.fbp.getFrameBuffer().convertZB2FB();
         this.fbp.getFrameBuffer().setViewport(0, 0, zBuf);
      }
*/
      fbp.update();
      repaint();
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // Define initial dimensions for a FrameBuffer.
      int width  = 512;
      int height = 512;
      // Create an InteractiveFrame containing a FrameBuffer
      // with the given dimensions. NOTE: We need to call the
      // TestCulling constructor in the Java GUI Event
      // Dispatch Thread, otherwise we get a race condition
      // between the constructor (running in the main() thread)
      // and the very first ComponentEvent (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         new Runnable() // an anonymous inner class constructor
         {
            public void run() // implement the Runnable interface
            {
               // call the constructor that builds the gui
               new TestCulling("Renderer 12", width, height);
            }
         }
      );
   }//main()


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, z/Z, keys to translate the models along the x, y, z axes.");
      System.out.println("Use the u/U, v/V, w/W, keys to rotate the models around the x, y, z axes.");
      System.out.println("Use the s/S keys to scale the size of the models.");
      System.out.println("Use the 'e' key to change the random colors.");
      System.out.println("Use the 'l' key to toggle letterboxing on and off.");
      System.out.println("Use the n/N keys to move the camera's near plane.");
      System.out.println("Use the o/O keys to change the size of the camera's view volume.");
      System.out.println("Use the r/R keys to change the aspect ratio of the camera's view volume.");
      System.out.println("Use the arrow keys to move the camera location left/right and up/down.");
      System.out.println("Use CTRL arrow keys to move the look-at point left/right and up/down.");
      System.out.println("Use the 'm' key to toggle showing the Model & View transformation matrices.");
//      System.out.println("Use the ' ' key to toggle between the image buffer and z-buffer.");
      System.out.println("Use the 'g' key to toggle back face culling.");
      System.out.println("Use the 'G' key to toggle CW vs CCW.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
