/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   This example shows that a triangle may be
   clipped into as many as ten new triangles.

   If you run this program you get four images of
   a red polygon that is the result of a triangle
   that has been clipped along three or four edges.
   To see how many triangles the polygon is made of
   (that is, how many triangles the original triangle
   got clipped into) you need to make a modification
   to the renderer's triangle clipping code.

   Open the file ClipTriangle.java in the pipeline package.

   Just after line 304 (near the end of the
   interpolateNewTriangle() method), add this line of code.
      t_new.setColorRandom();

   Just after line 369 (near the end of the
   interpolateNewTriangles() method), add these two lines
   of  code.
      t1_new.setColorRandom();
      t2_new.setColorRandom();

   The new code gives the clipped triangles random
   colors so that we can see them in the final image.
   After changing ClipTriangle.java, recompile this
   file and run it (you may need to delete all the
   class files first).
*/
public class TriangleClippingDemo
{
   public static void main(String[] args)
   {
      // Create a Model object to hold the geometry.
      Model model = new Model();
      model.doBackFaceCulling = false;

      // Create geometry for the Model.
      Vertex v0 = new Vertex( 0.0,  1.5, -1.0);
      Vertex v1 = new Vertex( 1.5,  0.0, -1.0);
      Vertex v2 = new Vertex(-0.5, -1.5, -1.0);

      Vertex v3 = new Vertex( 0.0,  1.5, -1.0);
      Vertex v4 = new Vertex( 1.5,  0.0, -1.0);
      Vertex v5 = new Vertex(-2.5, -2.5, -1.0);

      model.addVertex(v0, v1, v2, v3, v4, v5);

      Triangle t1 = new Triangle(model, 0, 1, 2);
      t1.setColor( Color.red );

      Triangle t2 = new Triangle(model, 1, 0, 2);
      t2.setColor( Color.red );

      Triangle t3 = new Triangle(model, 3, 4, 5);
      t3.setColor( Color.red );

      Triangle t4 = new Triangle(model, 4, 3, 5);
      t4.setColor( Color.red );


      // Create a Scene object to hold the model.
      Scene scene = new Scene();

      // Add the model to the Scene.
      scene.addPosition(new Position(model));

      // Create a framebuffer to render our scene into.
      int vp_width  = 600;
      int vp_height = 600;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);

      // Turn on pipeline debugging so that we can get
      // a text representation of the clipping results.
      Pipeline.debug = true;

      // Render four times, once using each of the above four triangles.
      model.addPrimitive( t1 );
      fb.clearFB(java.awt.Color.lightGray);
      Pipeline.render(scene, fb);
      fb.dumpFB2File("TriangleClippingDemo_1.ppm");

      model.primitiveList.remove(0);
      model.addPrimitive( t2 );
      fb.clearFB(java.awt.Color.lightGray);
      Pipeline.render(scene, fb);
      fb.dumpFB2File("TriangleClippingDemo_2.ppm");

      model.primitiveList.remove(0);
      model.addPrimitive( t3 );
      fb.clearFB(java.awt.Color.lightGray);
      Pipeline.render(scene, fb);
      fb.dumpFB2File("TriangleClippingDemo_3.ppm");

      model.primitiveList.remove(0);
      model.addPrimitive( t4 );
      fb.clearFB(java.awt.Color.lightGray);
      Pipeline.render(scene, fb);
      fb.dumpFB2File("TriangleClippingDemo_4.ppm");
   }
}
