/*

*/

package renderer.models_F;
import  renderer.scene.*;

/**
   Create a wireframe model of a cube aligned with
   the x, y, and z axes and with one corner at the
   origin.
<p>
   Here is a picture showing how the cube's eight vertices
   are labeled.
<pre>{@code
                  v[4]
                   +-----------------+ v[5]
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
      v[7] +-----------------+ v[6]  |
           |       |         |       |               y
           |       |         |       |               |
           |       |         |       |               |
           |  v[0] +---------|-------+ v[1]          |
           |      /          |      /                |
           |    /            |    /                  +----. x
           |  /              |  /                   /
           |/                |/                    /
           +-----------------+                    /
          v[3]              v[2]                 z
}</pre>

   @see Cube
*/
public class Box extends Model
{
   public Box(double xs, double ys, double zs)
   {
      super();

      // Create the cube's geometry.
      Vertex[] v = new Vertex[8];
      v[0] = new Vertex(0,    0,    0);
      v[1] = new Vertex(0+xs, 0,    0);
      v[2] = new Vertex(0+xs, 0+ys, 0);
      v[3] = new Vertex(0,    0+ys, 0);
      v[4] = new Vertex(0,    0+ys, 0+zs);
      v[5] = new Vertex(0+xs, 0+ys, 0+zs);
      v[6] = new Vertex(0+xs, 0,    0+zs);
      v[7] = new Vertex(0,    0,    0+zs);

      // Create six square faces.
      // Make sure the faces are all
      // oriented in the same way!
      // Remember that front facing
      // should be counter-clockwise
      // (and back facing is clock-wise).

      // front face
      this.addFace(v[5], v[4], v[7], v[6]);
      // top face
      this.addFace(v[5], v[2], v[3], v[4]);
      // right side
      this.addFace(v[5], v[6], v[1], v[2]);
      // back face
      this.addFace(v[0], v[3], v[2], v[1]);
      // bottom face
      this.addFace(v[0], v[1], v[6], v[7]);
      // left side
      this.addFace(v[0], v[7], v[4], v[3]);
   }
}//Box
