/*

*/

package renderer.models_F;
import  renderer.scene.*;

/**
   Create a solid model of a cube with its center
   at the origin, having edge length 2, and with its
   vertices at {@code (1, 1, 1)}.
<p>
   Here is a picture showing how the cube's eight vertices
   are labeled.
<pre>{@code
                  v[4]
                   +-----------------+ v[5]
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
      v[7] +-----------------+ v[6]  |
           |       |         |       |               y
           |       |         |       |               |
           |       |         |       |               |
           |  v[0] +---------|-------+ v[1]          |
           |      /          |      /                |
           |    /            |    /                  +----. x
           |  /              |  /                   /
           |/                |/                    /
           +-----------------+                    /
          v[3]              v[2]                 z
}</pre>

   @see Tetrahedron
   @see Octahedron
   @see Icosahedron
   @see Dodecahedron
*/
public class Cube extends Model
{
   /**
      Create a cube with its center at the origin, having edge
      length 2, and with its vertices at {@code (1, 1, 1)}.
   */
   public Cube( )
   {
      super();

      // Create the cube's geometry.
      Vertex[] v = new Vertex[8];
      v[0] = new Vertex(-1, -1, -1); // four vertices around the bottom face
      v[1] = new Vertex( 1, -1, -1);
      v[2] = new Vertex( 1, -1,  1);
      v[3] = new Vertex(-1, -1,  1);
      v[4] = new Vertex(-1,  1, -1); // four vertices around the top face
      v[5] = new Vertex( 1,  1, -1);
      v[6] = new Vertex( 1,  1,  1);
      v[7] = new Vertex(-1,  1,  1);

      // Create the 6 faces of the cube.
      // bottom face
      addFace(v[0], v[1], v[2], v[3]);
      // front face
      addFace(v[3], v[2], v[6], v[7]);
      // right face
      addFace(v[2], v[1], v[5], v[6]);
      // back face
      addFace(v[1], v[0], v[4], v[5]);
      // left face
      addFace(v[0], v[3], v[7], v[4]);
      // top face
      addFace(v[7], v[6], v[5], v[4]);
   }
}//Cube
