/*

*/

package renderer.models_F;
import  renderer.scene.*;

/**
   Create a wireframe model of a regular octahedron
   with its center at the origin, having side length
   {@code  sqrt(2) = 1.4142},with its center plane given
   by the four vertices {@code  (1, 0, 1)}. and with
   the top and bottom vertices being {@code  (0, 1, 0)}.
<p>
   See <a href="http://en.wikipedia.org/wiki/Octahedron" target="_top">
                http://en.wikipedia.org/wiki/Octahedron</a>

   @see Tetrahedron
   @see Cube
   @see Icosahedron
   @see Dodecahedron
*/
public class Octahedron extends Model
{
   /**
      Create a regular octahedron with its center at the
      origin, having side length {@code  sqrt(2) = 1.4142},
      with its center plane given by the four vertices
      {@code  (1, 0, 1)}. and with the top and bottom
      vertices being {@code  (0, 1, 0)}.
   */
   public Octahedron()
   {
      super();

      // Create the octahedron's geometry.
      double sqrt3 = Math.sqrt(3.0);
      double sqrt2 = Math.sqrt(2.0);
      Vertex[] v = new Vertex[6];
      v[0] = new Vertex( 0.5, 0,  0.5);  // four vertices around the center plane
      v[1] = new Vertex(-0.5, 0,  0.5);
      v[2] = new Vertex(-0.5, 0, -0.5);
      v[3] = new Vertex( 0.5, 0, -0.5);
      v[4] = new Vertex( 0,  1/sqrt2, 0);   // vertex at the top
      v[5] = new Vertex( 0, -1/sqrt2, 0);   // vertex at the bottom

      // Create eight triangular faces.
      // Make sure the faces are all
      // oriented in the same way!
      // Remember that front facing
      // should be counter-clockwise
      // (and back facing is clock-wise).
      addFace(v[4], v[1], v[0]);
      addFace(v[4], v[0], v[3]);
      addFace(v[4], v[2], v[1]);
      addFace(v[4], v[3], v[2]);
      addFace(v[5], v[0], v[1]);
      addFace(v[5], v[3], v[0]);
      addFace(v[5], v[1], v[2]);
      addFace(v[5], v[2], v[3]);
   }
}//Octahedron
