/*

*/

package renderer.models_T;
import  renderer.scene.*;

/**
   Create a solid model of a regular octahedron
   with its center at the origin, having side length
   {@code  sqrt(2) = 1.4142},with its center plane given
   by the four vertices {@code  (1, 0, 1)}. and with
   the top and bottom vertices being {@code  (0, 1, 0)}.
<p>
   See <a href="http://en.wikipedia.org/wiki/Octahedron" target="_top">
                http://en.wikipedia.org/wiki/Octahedron</a>

   @see Tetrahedron
   @see Cube
   @see Icosahedron
   @see Dodecahedron
*/
public class Octahedron extends Model
{
   /**
      Create a regular octahedron with its center at the
      origin, having side length {@code  sqrt(2) = 1.4142},
      with its center plane given by the four vertices
      {@code  (1, 0, 1)}. and with the top and bottom
      vertices being {@code  (0, 1, 0)}.
   */
   public Octahedron()
   {
      super();

      // Create the octahedron's geometry.
      // It has 6 vertices, 12 edges, and 8 faces.
      Vertex[] v = new Vertex[6];
      v[0] = new Vertex( 1,  0,  0);  // four vertices around the center plane
      v[1] = new Vertex( 0,  0, -1);
      v[2] = new Vertex(-1,  0,  0);
      v[3] = new Vertex( 0,  0,  1);
      v[4] = new Vertex( 0,  1,  0);   // vertex at the top
      v[5] = new Vertex( 0, -1,  0);   // vertex at the bottom
/*
      // These vertices create an Octahedron with side length 1.
      double sqrt3 = Math.sqrt(3.0);
      double sqrt2 = Math.sqrt(2.0);
      Vertex[] v = new Vertex[6];
      v[0] = new Vertex( 0.5, 0,  0.5);  // four vertices around the center plane
      v[1] = new Vertex(-0.5, 0,  0.5);
      v[2] = new Vertex(-0.5, 0, -0.5);
      v[3] = new Vertex( 0.5, 0, -0.5);
      v[4] = new Vertex( 0,  1/sqrt2, 0);   // vertex at the top
      v[5] = new Vertex( 0, -1/sqrt2, 0);   // vertex at the bottom
*/
      // Create 8 triangles.
      // face 1
      addTriangle( v[0], v[1], v[4] );
      // face 2
      addTriangle( v[1], v[2], v[4] );
      // face 3
      addTriangle( v[2], v[3], v[4] );
      // face 4
      addTriangle( v[3], v[0], v[4] );

      // face 5
      addTriangle( v[1], v[0], v[5] );
      // face 6
      addTriangle( v[2], v[1], v[5] );
      // face 7
      addTriangle( v[3], v[2], v[5] );
      // face 8
      addTriangle( v[0], v[3], v[5] );
   }
}//Octahedron
