/*

*/

package renderer.models_T;
import  renderer.scene.*;

/**
   Create a solid model of a right square pyramid with its
   base in the xz-plane and its apex on the positive y-axis.
<p>
   See <a href="https://en.wikipedia.org/wiki/Pyramid_(geometry)" target="_top">
                https://en.wikipedia.org/wiki/Pyramid_(geometry)</a>

   @see PyramidFrustum
*/
public class Pyramid extends Model
{
   /**
      Create a right square pyramid with its base in the xz-plane,
      a base side length of 2, height 1, and apex on the positive y-axis.
   */
   public Pyramid( )
   {
      this(2, 1, 15, 4);
   }


   /**
      Create a right square pyramid with its base in the xz-plane,
      a base length of {@code s}, height {@code h}, and apex on the
      positive y-axis.

      @param s  side length of the base in the xz-plane
      @param h  height of the apex on the y-axis
      @param n  number of lines of latitude around the body of the pyramid
      @param k  number of triangles in the triangle fan at the top of each side
   */
   public Pyramid(double s, double h, int n, int k)
   {
      super();

      if (n < 1) n = 1;
      if (k < 1) k = 1;

      // Create the pyramid's geometry.

      // An array of vertices to be used to create triangles.
      Vertex[] v = new Vertex[5];

      v[0] = new Vertex( 0, h,  0);
      v[1] = new Vertex(-s/2, 0, -s/2);
      v[2] = new Vertex(-s/2, 0,  s/2);
      v[3] = new Vertex( s/2, 0, -s/2);
      v[4] = new Vertex( s/2, 0,  s/2);

      // Create 6 triangles.
      addTriangle( v[0], v[1], v[2] ); // 4 sides
      addTriangle( v[0], v[2], v[4] );
      addTriangle( v[0], v[4], v[3] );
      addTriangle( v[0], v[3], v[1] );
      addTriangle( v[1], v[3], v[2] ); // 2 base triangles
      addTriangle( v[2], v[3], v[4] );
   }
}//Pyramid
