/*

*/

package renderer.pipeline;
import  renderer.scene.*;
import  renderer.framebuffer.*;

import java.awt.Color;

/**
   Rasterize a {@link Point} into pixels in the
   {@link FrameBuffer}'s viewport.
*/
public class RasterizePoint
{
   /**
      Rasterize a {@link Point} into pixels in the
      {@link FrameBuffer}'s viewport.

      @param pt  {@code Point} to rasterize into the {@code FrameBuffer}
      @param fb  {@code FrameBuffer} to hold rasterized pixels
   */
   public static void rasterize(Primitive pt, FrameBuffer fb)
   {
      boolean debug = Rasterize.debug;

      int w = fb.getWidthVP();
      int h = fb.getHeightVP();

      for (int index : pt.indexList)
      {
         Vertex v = pt.model.vertexList.get(index);

         // Get the color from the vertex.
         double r = v.r;
         double g = v.g;
         double b = v.b;

         // Round the point's coordinates to the nearest logical pixel.
         double x = Math.round( v.x );
         double y = Math.round( v.y );

         fb.setPixelVP( (int)x - 1,
                        h - (int)y,
                        new Color((float)r, (float)g, (float)b) );

         // log interesting information to standard output
         if (debug) logPixel(w, h, x, y, r, g, b);
      }
   }

   private static void logPixel(int w, int h, double x, double y, double r, double g, double b)
   {
      System.out.printf("[w=%d, h=%d]  x=%f, y=%f, r=%f, g=%f, b=%f\n", w, h, x, y, r, g, b);
   }
}
