/*

*/

package renderer.scene;

import java.util.List;
import java.util.ArrayList;
import java.awt.Color;

/**
   A {@code LineSegment} object has two integers that
   represent the endpoints of the line segment. The
   integers are indices into the {@link Vertex} list
   of a {@link Model} object.
*/
public class LineSegment extends Primitive
{
   /**
      Construct a {@code LineSegment} object with two integer indices
      into a {@link Model}'s list of {@link Vertex} objects.

      @param model  {@link Model} containing the {@link Vertex} list for the new {@code LineSegment}
      @param i0     index of 1st endpoint of the new {@code LineSegment}
      @param i1     index of 2nd endpoint of the new {@code LineSegment}
   */
   public LineSegment(Model model, int i0, int i1)
   {
      super(model);

      indexList.add(i0);
      indexList.add(i1);
   }


   /**
      Construct a {@code LineSegment} from a {@link List} of two indices.

      @param model  {@link Model} containing the {@link Vertex} list for the new {@code LineSegment}
      @param vList  {@code List} of vertices to put into the new {@code Primitive}
   */
   protected LineSegment(Model model, List<Integer> iList)
   {
      super(model, iList);
   }


   /**
      Construct a {@code LineSegment} from an array of two indices.

      @param model   {@link Model} containing the {@link Vertex} list for the new {@code LineSegment}
      @param vArray  array of vertices to put into the new {@code Primitive}
   */
   protected LineSegment(Model model, Integer... iArray)
   {
      super(model, iArray);
   }


   /**
      Implement the makeCopy() method from the abstract base class.
      <p>
      Make a deep copy of this {@code LineSegment} primitive.

      @param newModel  the {@link Model} that the new {@code LineSegment} will be part of
      @return a {@code LineSegent} object that is a deep copy of this {@code LineSegent}
   */
   public LineSegment makeCopy(Model newModel)
   {
      List<Integer> newIndexList = new ArrayList<>(this.indexList.size());
      for (Integer index : this.indexList)
      {
         newIndexList.add(index);
      }
      return new LineSegment(newModel, newIndexList);
   }


   /**
      For debugging.

      @return {@link String} representation of this {@code LineSegment} object
   */
   @Override
   public String toString()
   {
      return "Line Segment: (" + indexList.get(0) + ", "
                               + indexList.get(1) + ")\n";
   }
}
