/*

*/

package renderer.scene;

import java.util.List;
import java.util.ArrayList;

/**
   A {@code Triangle} object has a list of three integers that
   represent the three corners of a triangle in space. The
   integers are indices into the {@link Vertex} list of a
   {@link Model} object.
*/
public class Triangle extends Primitive
{
   /**
      Construct a {@code Triangle} object with three integer indices
      into a {@link Model}'s list of {@link Vertex} objects.

      @param model  {@link Model} containing the {@link Vertex} list for the new {@code Triangle}
      @param i0     index of 1st {@link Vertex} of the new {@code Triangle}
      @param i1     index of 2nd {@link Vertex} of the new {@code Triangle}
      @param i3     index of 2nd {@link Vertex} of the new {@code Triangle}
   */
   public Triangle(Model model, int i0, int i1, int i2)
   {
      super(model);

      indexList.add(i0);
      indexList.add(i1);
      indexList.add(i2);
   }


   /**
      Construct a default (empty) {@code Triangle}.

      @param model  {@link Model} containing the {@link Vertex} list for the new {@code Triangle}
   */
   public Triangle(Model model)
   {
      super(model);
   }


   /**
      Construct a {@code Triangle} from an array of integer indices.

      @param model   {@link Model} containing the {@link Vertex} list for the new {@code Triangle}
      @param iArray  array of vertices to put into the new {@code Triangle}
   */
   public Triangle(Model model, Integer... iArray)
   {
      super(model, iArray);
   }


   /**
      Construct a {@code Triangle} from a {@link List} of integer indices.

      @param model  {@link Model} containing the {@link Vertex} list for the new {@code Triangle}
      @param iList  {@link List} of vertices to put into the new {@code Triangle}
   */
   public Triangle(Model model, List<Integer> iList)
   {
      super(model, iList);
   }


   /**
      Implement the makeCopy() method from the abstract base class.
      <p>
      Make a deep copy of this {@code Triangle} primitive.

      @param newModel  the {@link Model} that the new {@code Triangle} will be part of
      @return a {@code Triangle} object that is a deep copy of this {@code Triangle}
   */
   public Triangle makeCopy(Model newModel)
   {
      List<Integer> newIndexList = new ArrayList<>(this.indexList.size());
      for (Integer index : this.indexList)
      {
         newIndexList.add(index);
      }
      return new Triangle(newModel, newIndexList);
   }


   /**
      For debugging.

      @return {@link String} representation of this {@code Triangle} object
   */
   @Override
   public String toString()
   {
      String result = "Triangle: (";
      for (Integer index : this.indexList)
      {
         result += index + ", ";
      }
      return result + ")\n";
   }
}
